"""How to generate code:
    - memory size is constsnt and initiall value of each container
        is chosen randomly (there are 17 possibilities)
    - when making new node there is some probability that it will be
        the last node if there are already more than minimally needed
        number of nodes
    - for each node its type is randomly chosen
    - if node type is Mover, randomly chose containers (state container
        is more probable)
    - after generating all nodes inexts are radnomly filled
    - in the end it is checked for all jumpspots whether there is Waiter
        after them and before next IfNode
"""

import random

if __name__ == "__main__":
    import logic
else:
    from . import logic


MEMORY_SIZE = 8
STATE_PROBABILITY = 2
MAX_WAIT = 3
END_PROBABILITY = 10
MIN_NODES = 20
MAX_NODES = 30


def random_values():
    """
    attack = random.randrange(2)
    if attack:
        n = random.randrange(8)
        if n < 4:
            return (n % 3 - 1, n // 3 -1, 1)
        n += 1
        return (n % 3 - 1, n // 3 -1, 1)
    else:
        return (random.randint(-1, 1), random.randint(-1, 1), 0)
    """
    return (random.randint(-1, 1), random.randint(-1, 1))


class Generator:
    NODES = [logic.IfNode, logic.Mover, logic.Waiter, logic.Add, logic.Rotator]
    def __init__(self):
        self.state = logic.Container(random_values())
        self.state.set_name("state")
        self.memory = [logic.Container(random_values()) for x in
                       range(MEMORY_SIZE)]
        for i, mem in enumerate(self.memory):
            mem.set_name("memory No. {}".format(i))
        #self.nodes = set()
        self.num = 0

    def make_new_node(self):
        class_ = random.choice(self.NODES)
        args = [self.last]
        for arg in class_.args:
            arg_type, *opts = arg
            if arg_type == logic.Container:
                if "equal" in opts:
                    n = random.randrange(MEMORY_SIZE + 1)
                    if n:
                        args.append(self.memory[n - 1])
                    else:
                        args.append(self.state)
                else:
                    if random.randrange(STATE_PROBABILITY):
                        n = random.randrange(MEMORY_SIZE)
                        args.append(self.memory[n])
                    else:
                        args.append(self.state)
                if "different" in opts:
                    while args[-2] == args[-1]:
                        if "equal":
                            n = random.randrange(MEMORY_SIZE + 1)
                            if n:
                                args[-1] = self.memory[n - 1]
                            else:
                                args[-1] = self.state
                        else:
                            if random.randrange(STATE_PROBABILITY):
                                n = random.randrange(MEMORY_SIZE)
                                args[-1] = self.memory[n]
                            else:
                                args[-1] = self.state
            elif arg_type == int:
                args.append(random.randrange(MAX_WAIT) + 1)
            elif arg_type == str:
                args.append(random.choice(("L", "R")))
        node = class_(*args)
        #self.nodes.add(node)
        #self.mark_node(node)
        #print(":a", node.mark)
        #if node.prev is not None and node.prev.next is None:
        #    print("HERE!")
        return node
    
    def mark_node(self, node):
        node.mark = self.num
        self.num += 1

    def set_waiter(self, node):
        n = 1
        while not isinstance(node, logic.Waiter) and node is not None:
            if isinstance(node, logic.IfNode):
                x = random.randrange(n)
                for i in range(x):
                    node = node.prev
                new = logic.Waiter(node.prev)
                #self.mark_node(new)
                #print("|||:>", node.prev.mark)
                #print(":b", node.mark, new.mark)
                node.prev = new
                new.next = node
                return node
            node = node.next
        return node

    def get_after(self, node, n):
        for i in range(n):
            node = node.next
        return node

    def generate(self):
        self.last = None
        initial = self.make_new_node()
        self.last = initial
        n = 1
        while ((n < MIN_NODES or random.randrange(END_PROBABILITY)) and
                n < MAX_NODES):
            self.last = self.make_new_node()
            n += 1

        """node = initial
        while node is not None:
            print(node.mark)
            node = node.next
        print(n)"""
        node = initial
        while node is not None:
            if isinstance(node, logic.IfNode):
                x = random.randrange(n)
                node2 = self.get_after(initial, x)
                while isinstance(node2, logic.IfNode):
                    x = random.randrange(n)
                    node2 = self.get_after(initial, x)
                #print("xxx", x, type(node2))
                node.set_inext(node2)
            node = node.next

        node = initial
        node.jumpspot = True
        #print("OK")
        while node is not None:
            if node.jumpspot:
                node = self.set_waiter(node)
            if node is not None:
                node = node.next
        #print("OK")
        node = initial
        while node.prev is not None:
            node = node.prev
        initial = node

        x = 1
        while node is not None:
            node.set_line(x)
            x += 1
            #self.nodes.remove(node)
            node = node.next

        #for node in self.nodes:
        #    print(node.mark)

        return initial


if __name__ == "__main__":
    gen = Generator()
    node = gen.generate()
    while node is not None:
        print(node)
        node = node.next
