import random

import pygame

from . import code_gen, logic, tools

COLORKEY = (254, 12, 197)  # very dangerous color
if __name__ == "__main__":
    FONT_PATH = "../art/font.ttf"
else:
    FONT_PATH = tools.get_path("font", type_="font")[0]


class Cont:
    def __init__(self, screen, cont, state, x, y, w):
        self.screen = screen
        self.cont = cont
        self.cont.graph = self
        print(self.cont, "=", self.cont.state)
        self.state = state
        self.x = x
        self.y = y
        self.w = w
        self.h = w
        self.pic = pygame.Surface((int(self.w), int(self.h)))
        self.pic.set_colorkey(COLORKEY)
        self.update()

    def update(self):
        self.pic.fill(COLORKEY)
        points = [(self.w / 2, self.h / 2), (self.w / 2 + self.w *
                                             self.cont.state[0] / 2,
                                             self.h / 2 + self.h *
                                             self.cont.state[1] / 2)]
        pygame.draw.line(self.pic, (0, 0, 0), points[0], points[1])
        if any(self.cont.state):
            dx = points[-2][0] - points[-1][0]
            dy = points[-2][1] - points[-1][1]
            mult = 20 / tools.dis(points[-2], points[-1])
            for angle in (-30, 30):
                ndx, ndy = tools.rotate(dx, dy, angle)
                ndx *= mult
                ndy *= mult
                point = (points[-1][0] + ndx, points[-1][1] + ndy)
                pygame.draw.line(self.pic, (0, 0, 0), points[-1], point)

        pygame.draw.rect(self.pic, (255 if self.state else 0, 0, 0),
                         pygame.Rect(0, 0, self.w - 1, self.h - 1), 2)

    def blit(self):
        self.screen.blit(self.pic, (self.x, self.y))


class Node:
    def __init__(self, screen, node, x, y, w, font):
        self.screen = screen
        self.node = node
        self.node.graph = self

        self.surface = pygame.Surface(screen.get_size())
        self.surface.set_colorkey(COLORKEY)
        self.surface.fill(COLORKEY)
        self.x = x
        self.y = y
        self.w = w
        self.h = w
        self.font = font

    def generate(self):
        self.rect = pygame.Surface((self.w, self.h))
        self.rect.fill(self.node.color)
        props = self.node.visual()
        if "text" in props:
            text = self.font.render(props["text"], False, (0, 0, 0))
            x = self.x + self.w / 2 - text.get_width() / 2
            y = self.y + self.h / 2 - text.get_height() / 2
            self.surface.blit(text, (x, y))
        if "arrows to" in props:
            for node in props["arrows to"]:
                self.arrow_to(node.graph, False)
        if "c. arrows to" in props:
            for cont in props["c. arrows to"]:
                self.arrow_to(cont.graph, True)
        if "c. arrows from" in props:
            for cont in props["c. arrows from"]:
                self.arrow_to(cont.graph, True, True)

        self.deactivate()

    def arrow_to(self, node, is_cont, rev=False):
        x1 = self.x + self.w / 2
        x3 = node.x + node.w / 2
        if rev:
            x1, x3 = x3, x1
        if is_cont:
            y1 = self.y + self.h
            y3 = node.y 
            y2 = (y1 + y3) / 2
            x2 = x1
        else:
            y1 = y3 = self.y
            x2 = (x1 + x3) / 2
            y2 = max(self.y - abs(x1 - x3) * 2 + random.randint(-50, 50),
                     -self.y + 25)
        if rev:
            y1, y3 = y3, y1
        points = tools.bezier(x1, y1, x2, y2, x3, y3, 61)
        if is_cont:
            for n in range(31):
                pygame.draw.lines(self.surface, (0, 0, 0), False,
                                  points[2 * n:2 + 2 * n])
        else:
            pygame.draw.lines(self.surface, (0, 0, 0), False, points)
        dx = points[-2][0] - points[-1][0]
        dy = points[-2][1] - points[-1][1]
        mult = 20 / tools.dis(points[-2], points[-1])
        for angle in (-30, 30):
            ndx, ndy = tools.rotate(dx, dy, angle)
            ndx *= mult
            ndy *= mult
            point = (points[-1][0] + ndx, points[-1][1] + ndy)
            pygame.draw.line(self.surface, (0, 0, 0), points[-1], point)

    def activate(self):
        self.rect.set_alpha(None)
        self.surface.set_alpha(None)

    def deactivate(self):
        self.rect.set_alpha(100)
        self.surface.set_alpha(100)

    def blit_rect(self):
        self.screen.blit(self.rect, (self.x, self.y))

    def blit_surface(self):
        self.screen.blit(self.surface, (0, 0))


class Mind:
    def __init__(self, screen):
        self.screen = screen
        self.ratio = tools.get_font_ratio(FONT_PATH)

        self.gen = code_gen.Generator()
        self.initial = self.gen.generate()
        self.len = 0
        node = self.initial
        while node is not None:
            print(node)
            node = node.next
            self.len += 1
        # 6/5 x * l + 1/5 x = w
        # 6lx + x = 5w
        # (6l + 1)x = 5w
        # x = 5w / (6l + 1)
        self.node_w = self.screen.get_width() * 5 / (6 * self.len + 1)
        self.node_y = self.screen.get_height() / 3
        self.font = pygame.font.Font(FONT_PATH,
                                     int(self.ratio * (self.node_w - 2)))
        self.nodes = []
        node = self.initial
        n = 0
        while node is not None:
            self.nodes.append(Node(self.screen, node, self.node_w / 5 + n *
                                   self.node_w * 1.2, self.node_y,
                                   self.node_w, self.font))
            node = node.next
            n += 1

        self.cont_w = (self.screen.get_width() * 5 /
                       (6 * code_gen.MEMORY_SIZE + 7))
        self.cont_y = self.screen.get_height() * 0.95 - self.cont_w
        self.conts = []
        for n in range(code_gen.MEMORY_SIZE + 1):
            if n < code_gen.MEMORY_SIZE / 2:
                self.conts.append(Cont(self.screen, self.gen.memory[n],
                                       False, self.cont_w / 5 + n *
                                       self.cont_w * 1.2, self.cont_y,
                                       self.cont_w))
            elif n == code_gen.MEMORY_SIZE / 2:
                self.conts.append(Cont(self.screen, self.gen.state, True,
                                       self.cont_w / 5 + n * self.cont_w
                                       * 1.2, self.cont_y, self.cont_w))
            else:
                self.conts.append(Cont(self.screen, self.gen.memory[n - 1],
                                       False, self.cont_w / 5 + n *
                                       self.cont_w * 1.2, self.cont_y,
                                       self.cont_w))
        
        for node in self.nodes:
            node.generate()
        self.current = None

    def blit(self):
        for cont in self.conts:
            cont.blit()
        for node in self.nodes:
            node.blit_rect()
        for node in self.nodes:
            node.blit_surface()

    def click(self, x, y):
        if (self.node_y < y < self.node_y + self.node_w and
            x % int(self.node_w * 6 / 5) > self.node_w / 5):
            if self.current:
                self.current.deactivate()
            self.current = self.nodes[int(x / (self.node_w * 6 / 5))]
            self.current.activate()
        elif self.current:
            self.current.deactivate()
            self.current = None

if __name__ == "__main__":
    screen = pygame.display.set_mode((800, 500))
    
    pygame.init()
    
    m = Mind(screen)
    run = True
    while run:
        for event in pygame.event.get():
            if event.type == pygame.QUIT:
                run = False
            if event.type == pygame.MOUSEBUTTONDOWN:
                m.click(*event.pos)

        screen.fill((255, 255, 255))

        m.blit()

        pygame.display.flip()
