import random

import pygame

if __name__ == "__main__":
    import logic
    import code_gen
    PLAYER_PATH_1 = "../art/player.png"
    PLAYER_PATH_2 = "../art/playerd.png"
    TILE_PATH = "../art/tile.png"
    HUMAN_PATH = "../art/human.png"
else:
    from . import logic, tools
    PLAYER_PATH_1, PLAYER_PATH_2, TILE_PATH, HUMAN_PATH, DEAD_PATH =\
            tools.get_path("nplayer", "nplayerd", "tile", "human", "dead")


TILE_W = 30
TILE_H = 30
COLORKEY = (254, 12, 197)  # very dangerous color
BG_COLOR = (127, 51, 0)


class Player:
    UPDATE_TIME = 1000
    def __init__(self, screen, x, y, map_, execute, state):
        self.screen = screen
        self.x = x
        self.y = y
        self.dir = 0
        self.map = map_
        self.execute = execute
        self.state = state
        self.image = (pygame.image.load(PLAYER_PATH_1),
                      pygame.image.load(PLAYER_PATH_2))
        self.images = [pygame.transform.rotate(self.image[x % 2], -90 * (x // 2))
                       for x in range(8)]
        #self.next_update_time = pygame.time.get_ticks() + self.UPDATE_TIME
        self.killed = 0

    @classmethod
    def from_map(cls, map_, execute, state):
        x = random.randrange(map_.w)
        y = random.randrange(map_.h)
        while map_.tiles[y][x]:
            x = random.randrange(map_.w)
            y = random.randrange(map_.h)
        return cls(map_.screen, x, y, map_, execute, state)

    def blit(self):
        self.screen.blit(self.images[self.dir], (self.x * TILE_W, self.y * TILE_H))
        """
        if pygame.time.get_ticks() > self.next_update_time:
            self.update()
            self.next_update_time = pygame.time.get_ticks() + self.UPDATE_TIME
        """

    def update(self):
        x, y = self.state.state
        self.x += x
        self.x %= self.map.w
        self.y += y
        self.y %= self.map.h
        self.dir = 0
        if x:
            self.dir += 6 if x == -1 else 2
            self.dir += -y if x == -1 else y
        elif y == 1:
            self.dir = 4
        if self.map.tiles[self.y][self.x] == 1:
            self.killed += 1
            self.map.tiles[self.y][self.x] = 2
            self.map.refresh(self.x, self.y)
        next(self.execute)


class Map:
    def __init__(self, screen, w, h, people):
        self.screen = screen
        self.w = w
        self.h = h
        self.tiles = [[0 for x in range(w)] for y in range(h)]
        n = 0
        while n < people:
            x = random.randrange(self.w)
            y = random.randrange(self.h)
            if not self.tiles[y][x]:
                self.tiles[y][x] = 1
                n += 1

        self.surface = pygame.Surface((w * TILE_W, h * TILE_H))
        self.surface.fill(COLORKEY)
        self.surface.set_colorkey(COLORKEY)
        self.images = [pygame.image.load(TILE_PATH),
                       pygame.image.load(HUMAN_PATH),
                       pygame.image.load(DEAD_PATH)]
        for y, row in enumerate(self.tiles):
            for x, tile in enumerate(row):
                self.surface.blit(self.images[tile], (x * TILE_W, y * TILE_H))

    def blit(self): 
        self.screen.blit(self.surface, (0, 0))

    def refresh(self, x, y):
        self.surface.blit(self.images[self.tiles[y][x]], (x * TILE_W, y * TILE_H))

    def reset(self):
        self.surface.fill(COLORKEY)
        for y, row in enumerate(self.tiles):
            for x, tile in enumerate(row):
                if tile == 2:
                    row[x] = 1
                    tile = 1
                self.surface.blit(self.images[tile], (x * TILE_W, y * TILE_H))




if __name__ == "__main__":
    """
    state = logic.Container((0, 0, 0))
    const_u = logic.Container((0, -1, 0))
    const_ur = logic.Container((1, -1, 0))
    const_r = logic.Container((1, 0, 0))
    const_dr = logic.Container((1, 1, 0))
    const_d = logic.Container((0, 1, 0))
    const_dl = logic.Container((-1, 1, 0))
    const_l = logic.Container((-1, 0, 0))
    const_ul = logic.Container((-1, -1, 0))
    nodes = []
    nodes.append(logic.Mover(None, const_u, state))
    nodes.append(logic.Waiter(nodes[-1]))
    nodes.append(logic.Mover(nodes[-1], const_ur, state))
    #nodes.append(logic.Waiter(nodes[-1]))
    nodes.append(logic.Mover(nodes[-1], const_r, state))
    nodes.append(logic.Waiter(nodes[-1]))
    nodes.append(logic.Mover(nodes[-1], const_dr, state))
    #nodes.append(logic.Waiter(nodes[-1]))
    nodes.append(logic.Mover(nodes[-1], const_d, state))
    nodes.append(logic.Waiter(nodes[-1]))
    nodes.append(logic.Mover(nodes[-1], const_dl, state))
    #nodes.append(logic.Waiter(nodes[-1]))
    nodes.append(logic.Mover(nodes[-1], const_l, state))
    nodes.append(logic.Waiter(nodes[-1]))
    #nodes.append(logic.Mover(nodes[-1], const_ul, state))
    #nodes.append(logic.Waiter(nodes[-1]))
    exe = logic.Executor(nodes[0])
    """

    gen = code_gen.Generator()
    node = gen.generate()
    exe = logic.Executor(node)
    while node is not None:
        print(node)
        node = node.next
    state = gen.state

    screen = pygame.display.set_mode((600, 600))
    
    pygame.init()

    m = Map(screen, 20, 20)
    player = Player(screen, 9, 9, m, logic.execute(exe), state)
    run = True
    while run:
        for event in pygame.event.get():
            if event.type == pygame.QUIT:
                run = False
            elif event.type == pygame.KEYDOWN:
                if event.key == pygame.K_ESCAPE:
                    run = False

        screen.fill(BG_COLOR)

        m.blit()

        player.blit()

        pygame.display.flip()

    pygame.quit()
