import math
from os import path

import pygame


PATH = path.join("doc", "art")


def get_font_ratio(fontpath):
    tmp_font = pygame.font.Font(fontpath, 100)
    width, height = tmp_font.size("a")
    return 100 / max(width, height)


def bezier(x1, y1, x2, y2, x3, y3, steps):
    dx1 = (x2 - x1) / steps
    dy1 = (y2 - y1) / steps
    dx2 = (x3 - x2) / steps
    dy2 = (y3 - y2) / steps
    ret = []
    for step in range(steps + 1):
        part = step / steps
        ret.append((x1 + (x2 - x1) * part, y1 + (y2 - y1) * part))
        x1 += dx1
        y1 += dy1
        x2 += dx2
        y2 += dy2
    return ret


def rotate(x, y, angle):
    angle = math.radians(angle)
    rx = x * math.cos(angle) - y * math.sin(angle)
    ry = x * math.sin(angle) + y * math.cos(angle)
    return rx, ry


def dis(point1, point2):
    return math.sqrt((point2[0] - point1[0]) ** 2 +
                     (point2[1] - point1[1]) ** 2)


def get_path(*names, type_="image"):
    if type_ == "font":
        return [path.join(PATH, name + '.ttf') for name in names]
    elif type_ == "image":
        return [path.join(PATH, name + '.png') for name in names]
